@@ -1,3 +1,5 @@
+from __future__ import print_function, division
+
 from sympy.core import S, C, sympify, cacheit
 from sympy.core.function import Function, ArgumentIndexError, _coeff_isneg
 
@@ -7,14 +9,23 @@ from sympy.functions.elementary.miscellaneous import sqrt
 ########################### HYPERBOLIC FUNCTIONS ##############################
 ###############################################################################
 
+
 class HyperbolicFunction(Function):
-    """Base class for hyperbolic functions. """
+    """
+    Base class for hyperbolic functions.
+
+    See Also
+    ========
+
+    sinh, cosh, tanh, coth
+    """
 
     unbranched = True
 
+
 class sinh(HyperbolicFunction):
-    """
-    The hyperbolic sine function, :math:`\\frac{exp(x) - exp(-x)}{2}`.
+    r"""
+    The hyperbolic sine function, `\frac{e^x - e^{-x}}{2}`.
 
     * sinh(x) -> Returns the hyperbolic sine of x
 
@@ -23,7 +34,6 @@ class sinh(HyperbolicFunction):
 
     cosh, tanh, asinh
     """
-    nargs = 1
 
     def fdiff(self, argindex=1):
         """
@@ -72,15 +82,15 @@ class sinh(HyperbolicFunction):
 
             if arg.func == acosh:
                 x = arg.args[0]
-                return sqrt(x-1) * sqrt(x+1)
+                return sqrt(x - 1) * sqrt(x + 1)
 
             if arg.func == atanh:
                 x = arg.args[0]
-                return x/sqrt(1-x**2)
+                return x/sqrt(1 - x**2)
 
             if arg.func == acoth:
                 x = arg.args[0]
-                return 1/(sqrt(x-1) * sqrt(x+1))
+                return 1/(sqrt(x - 1) * sqrt(x + 1))
 
     @staticmethod
     @cacheit
@@ -95,7 +105,7 @@ class sinh(HyperbolicFunction):
 
             if len(previous_terms) > 2:
                 p = previous_terms[-2]
-                return p * x**2 / (n*(n-1))
+                return p * x**2 / (n*(n - 1))
             else:
                 return x**(n) / C.factorial(n)
 
@@ -118,6 +128,30 @@ class sinh(HyperbolicFunction):
             re, im = self.args[0].as_real_imag()
         return (sinh(re)*C.cos(im), cosh(re)*C.sin(im))
 
+    def _eval_expand_complex(self, deep=True, **hints):
+        re_part, im_part = self.as_real_imag(deep=deep, **hints)
+        return re_part + im_part*S.ImaginaryUnit
+
+    def _eval_expand_trig(self, deep=True, **hints):
+        if deep:
+            arg = self.args[0].expand(deep, **hints)
+        else:
+            arg = self.args[0]
+        x = None
+        if arg.is_Add: # TODO, implement more if deep stuff here
+            x, y = arg.as_two_terms()
+        else:
+            coeff, terms = arg.as_coeff_Mul(rational=True)
+            if coeff is not S.One and coeff.is_Integer and terms is not S.One:
+                x = terms
+                y = (coeff - 1)*x
+        if x is not None:
+            return (sinh(x)*cosh(y) + sinh(y)*cosh(x)).expand(trig=True)
+        return sinh(arg)
+
+    def _eval_rewrite_as_tractable(self, arg):
+        return (C.exp(arg) - C.exp(-arg)) / 2
+
     def _eval_rewrite_as_exp(self, arg):
         return (C.exp(arg) - C.exp(-arg)) / 2
 
@@ -152,9 +186,10 @@ class sinh(HyperbolicFunction):
         import sage.all as sage
         return sage.sinh(self.args[0]._sage_())
 
+
 class cosh(HyperbolicFunction):
-    """
-    The hyperbolic cosine function, :math:`\\frac{exp(x) + exp(-x)}{2}`.
+    r"""
+    The hyperbolic cosine function, `\frac{e^x + e^{-x}}{2}`.
 
     * cosh(x) -> Returns the hyperbolic cosine of x
 
@@ -163,7 +198,6 @@ class cosh(HyperbolicFunction):
 
     sinh, tanh, acosh
     """
-    nargs = 1
 
     def fdiff(self, argindex=1):
         if argindex == 1:
@@ -171,12 +205,6 @@ class cosh(HyperbolicFunction):
         else:
             raise ArgumentIndexError(self, argindex)
 
-    def inverse(self, argindex=1):
-        """
-        Returns the inverse of this function.
-        """
-        return acosh
-
     @classmethod
     def eval(cls, arg):
         arg = sympify(arg)
@@ -205,17 +233,17 @@ class cosh(HyperbolicFunction):
                     return cls(-arg)
 
             if arg.func == asinh:
-                return sqrt(1+arg.args[0]**2)
+                return sqrt(1 + arg.args[0]**2)
 
             if arg.func == acosh:
                 return arg.args[0]
 
             if arg.func == atanh:
-                return 1/sqrt(1-arg.args[0]**2)
+                return 1/sqrt(1 - arg.args[0]**2)
 
             if arg.func == acoth:
                 x = arg.args[0]
-                return x/(sqrt(x-1) * sqrt(x+1))
+                return x/(sqrt(x - 1) * sqrt(x + 1))
 
     @staticmethod
     @cacheit
@@ -227,7 +255,7 @@ class cosh(HyperbolicFunction):
 
             if len(previous_terms) > 2:
                 p = previous_terms[-2]
-                return p * x**2 / (n*(n-1))
+                return p * x**2 / (n*(n - 1))
             else:
                 return x**(n)/C.factorial(n)
 
@@ -248,6 +276,30 @@ class cosh(HyperbolicFunction):
 
         return (cosh(re)*C.cos(im), sinh(re)*C.sin(im))
 
+    def _eval_expand_complex(self, deep=True, **hints):
+        re_part, im_part = self.as_real_imag(deep=deep, **hints)
+        return re_part + im_part*S.ImaginaryUnit
+
+    def _eval_expand_trig(self, deep=True, **hints):
+        if deep:
+            arg = self.args[0].expand(deep, **hints)
+        else:
+            arg = self.args[0]
+        x = None
+        if arg.is_Add: # TODO, implement more if deep stuff here
+            x, y = arg.as_two_terms()
+        else:
+            coeff, terms = arg.as_coeff_Mul(rational=True)
+            if coeff is not S.One and coeff.is_Integer and terms is not S.One:
+                x = terms
+                y = (coeff - 1)*x
+        if x is not None:
+            return (cosh(x)*cosh(y) + sinh(x)*sinh(y)).expand(trig=True)
+        return cosh(arg)
+
+    def _eval_rewrite_as_tractable(self, arg):
+        return (C.exp(arg) + C.exp(-arg)) / 2
+
     def _eval_rewrite_as_exp(self, arg):
         return (C.exp(arg) + C.exp(-arg)) / 2
 
@@ -256,11 +308,11 @@ class cosh(HyperbolicFunction):
 
     def _eval_rewrite_as_tanh(self, arg):
         tanh_half = tanh(S.Half*arg)**2
-        return (1+tanh_half)/(1-tanh_half)
+        return (1 + tanh_half)/(1 - tanh_half)
 
     def _eval_rewrite_as_coth(self, arg):
         coth_half = coth(S.Half*arg)**2
-        return (coth_half+1)/(coth_half-1)
+        return (coth_half + 1)/(coth_half - 1)
 
     def _eval_as_leading_term(self, x):
         arg = self.args[0].as_leading_term(x)
@@ -282,9 +334,10 @@ class cosh(HyperbolicFunction):
         import sage.all as sage
         return sage.cosh(self.args[0]._sage_())
 
+
 class tanh(HyperbolicFunction):
-    """
-    The hyperbolic tangent function, :math:`\\frac{sinh(x)}{cosh(x)}`.
+    r"""
+    The hyperbolic tangent function, `\frac{\sinh(x)}{\cosh(x)}`.
 
     * tanh(x) -> Returns the hyperbolic tangent of x
 
@@ -293,7 +346,6 @@ class tanh(HyperbolicFunction):
 
     sinh, cosh, atanh
     """
-    nargs = 1
 
     def fdiff(self, argindex=1):
         if argindex == 1:
@@ -338,11 +390,11 @@ class tanh(HyperbolicFunction):
 
             if arg.func == asinh:
                 x = arg.args[0]
-                return x/sqrt(1+x**2)
+                return x/sqrt(1 + x**2)
 
             if arg.func == acosh:
                 x = arg.args[0]
-                return sqrt(x-1) * sqrt(x+1) / x
+                return sqrt(x - 1) * sqrt(x + 1) / x
 
             if arg.func == atanh:
                 return arg.args[0]
@@ -358,12 +410,12 @@ class tanh(HyperbolicFunction):
         else:
             x = sympify(x)
 
-            a = 2**(n+1)
+            a = 2**(n + 1)
 
-            B = C.bernoulli(n+1)
-            F = C.factorial(n+1)
+            B = C.bernoulli(n + 1)
+            F = C.factorial(n + 1)
 
-            return a*(a-1) * B/F * x**n
+            return a*(a - 1) * B/F * x**n
 
     def _eval_conjugate(self):
         return self.func(self.args[0].conjugate())
@@ -382,9 +434,13 @@ class tanh(HyperbolicFunction):
         denom = sinh(re)**2 + C.cos(im)**2
         return (sinh(re)*cosh(re)/denom, C.sin(im)*C.cos(im)/denom)
 
+    def _eval_rewrite_as_tractable(self, arg):
+        neg_exp, pos_exp = C.exp(-arg), C.exp(arg)
+        return (pos_exp - neg_exp)/(pos_exp + neg_exp)
+
     def _eval_rewrite_as_exp(self, arg):
         neg_exp, pos_exp = C.exp(-arg), C.exp(arg)
-        return (pos_exp-neg_exp)/(pos_exp+neg_exp)
+        return (pos_exp - neg_exp)/(pos_exp + neg_exp)
 
     def _eval_rewrite_as_sinh(self, arg):
         return S.ImaginaryUnit*sinh(arg)/sinh(S.Pi*S.ImaginaryUnit/2 - arg)
@@ -415,13 +471,13 @@ class tanh(HyperbolicFunction):
         import sage.all as sage
         return sage.tanh(self.args[0]._sage_())
 
+
 class coth(HyperbolicFunction):
-    """
-    The hyperbolic tangent function, :math:`\\frac{cosh(x)}{sinh(x)}`.
+    r"""
+    The hyperbolic cotangent function, `\frac{\cosh(x)}{\sinh(x)}`.
 
     * coth(x) -> Returns the hyperbolic cotangent of x
     """
-    nargs = 1
 
     def fdiff(self, argindex=1):
         if argindex == 1:
@@ -466,11 +522,11 @@ class coth(HyperbolicFunction):
 
             if arg.func == asinh:
                 x = arg.args[0]
-                return sqrt(1+x**2)/x
+                return sqrt(1 + x**2)/x
 
             if arg.func == acosh:
                 x = arg.args[0]
-                return x/(sqrt(x-1) * sqrt(x+1))
+                return x/(sqrt(x - 1) * sqrt(x + 1))
 
             if arg.func == atanh:
                 return 1/arg.args[0]
@@ -488,10 +544,10 @@ class coth(HyperbolicFunction):
         else:
             x = sympify(x)
 
-            B = C.bernoulli(n+1)
-            F = C.factorial(n+1)
+            B = C.bernoulli(n + 1)
+            F = C.factorial(n + 1)
 
-            return 2**(n+1) * B/F * x**n
+            return 2**(n + 1) * B/F * x**n
 
     def _eval_conjugate(self):
         return self.func(self.args[0].conjugate())
@@ -510,9 +566,13 @@ class coth(HyperbolicFunction):
         denom = sinh(re)**2 + C.sin(im)**2
         return (sinh(re)*cosh(re)/denom, -C.sin(im)*C.cos(im)/denom)
 
+    def _eval_rewrite_as_tractable(self, arg):
+        neg_exp, pos_exp = C.exp(-arg), C.exp(arg)
+        return (pos_exp + neg_exp)/(pos_exp - neg_exp)
+
     def _eval_rewrite_as_exp(self, arg):
         neg_exp, pos_exp = C.exp(-arg), C.exp(arg)
-        return (pos_exp+neg_exp)/(pos_exp-neg_exp)
+        return (pos_exp + neg_exp)/(pos_exp - neg_exp)
 
     def _eval_rewrite_as_sinh(self, arg):
         return -S.ImaginaryUnit*sinh(S.Pi*S.ImaginaryUnit/2 - arg)/sinh(arg)
@@ -551,7 +611,6 @@ class asinh(Function):
 
     acosh, atanh, sinh
     """
-    nargs = 1
 
     def fdiff(self, argindex=1):
         if argindex == 1:
@@ -599,7 +658,7 @@ class asinh(Function):
             x = sympify(x)
             if len(previous_terms) >= 2 and n > 2:
                 p = previous_terms[-2]
-                return -p * (n-2)**2/(n*(n-1)) * x**2
+                return -p * (n - 2)**2/(n*(n - 1)) * x**2
             else:
                 k = (n - 1) // 2
                 R = C.RisingFactorial(S.Half, k)
@@ -614,10 +673,17 @@ class asinh(Function):
         else:
             return self.func(arg)
 
+    def inverse(self, argindex=1):
+        """
+        Returns the inverse of this function.
+        """
+        return sinh
+
     def _sage_(self):
         import sage.all as sage
         return sage.asinh(self.args[0]._sage_())
 
+
 class acosh(Function):
     """
     The inverse hyperbolic cosine function.
@@ -629,7 +695,6 @@ class acosh(Function):
 
     asinh, atanh, cosh
     """
-    nargs = 1
 
     def fdiff(self, argindex=1):
         if argindex == 1:
@@ -657,26 +722,26 @@ class acosh(Function):
 
         if arg.is_number:
             cst_table = {
-                S.ImaginaryUnit : C.log(S.ImaginaryUnit*(1+sqrt(2))),
-                -S.ImaginaryUnit : C.log(-S.ImaginaryUnit*(1+sqrt(2))),
-                S.Half       : S.Pi/3,
-                -S.Half      : 2*S.Pi/3,
-                sqrt(2)/2    : S.Pi/4,
-                -sqrt(2)/2   : 3*S.Pi/4,
-                1/sqrt(2)    : S.Pi/4,
-                -1/sqrt(2)   : 3*S.Pi/4,
-                sqrt(3)/2    : S.Pi/6,
-                -sqrt(3)/2   : 5*S.Pi/6,
-                (sqrt(3)-1)/sqrt(2**3) : 5*S.Pi/12,
-                -(sqrt(3)-1)/sqrt(2**3) : 7*S.Pi/12,
-                sqrt(2+sqrt(2))/2 : S.Pi/8,
-                -sqrt(2+sqrt(2))/2 : 7*S.Pi/8,
-                sqrt(2-sqrt(2))/2 : 3*S.Pi/8,
-                -sqrt(2-sqrt(2))/2 : 5*S.Pi/8,
-                (1+sqrt(3))/(2*sqrt(2)) : S.Pi/12,
-                -(1+sqrt(3))/(2*sqrt(2)) : 11*S.Pi/12,
-                (sqrt(5)+1)/4 : S.Pi/5,
-                -(sqrt(5)+1)/4 : 4*S.Pi/5
+                S.ImaginaryUnit: C.log(S.ImaginaryUnit*(1 + sqrt(2))),
+                -S.ImaginaryUnit: C.log(-S.ImaginaryUnit*(1 + sqrt(2))),
+                S.Half: S.Pi/3,
+                -S.Half: 2*S.Pi/3,
+                sqrt(2)/2: S.Pi/4,
+                -sqrt(2)/2: 3*S.Pi/4,
+                1/sqrt(2): S.Pi/4,
+                -1/sqrt(2): 3*S.Pi/4,
+                sqrt(3)/2: S.Pi/6,
+                -sqrt(3)/2: 5*S.Pi/6,
+                (sqrt(3) - 1)/sqrt(2**3): 5*S.Pi/12,
+                -(sqrt(3) - 1)/sqrt(2**3): 7*S.Pi/12,
+                sqrt(2 + sqrt(2))/2: S.Pi/8,
+                -sqrt(2 + sqrt(2))/2: 7*S.Pi/8,
+                sqrt(2 - sqrt(2))/2: 3*S.Pi/8,
+                -sqrt(2 - sqrt(2))/2: 5*S.Pi/8,
+                (1 + sqrt(3))/(2*sqrt(2)): S.Pi/12,
+                -(1 + sqrt(3))/(2*sqrt(2)): 11*S.Pi/12,
+                (sqrt(5) + 1)/4: S.Pi/5,
+                -(sqrt(5) + 1)/4: 4*S.Pi/5
             }
 
             if arg in cst_table:
@@ -708,7 +773,7 @@ class acosh(Function):
             x = sympify(x)
             if len(previous_terms) >= 2 and n > 2:
                 p = previous_terms[-2]
-                return p * (n-2)**2/(n*(n-1)) * x**2
+                return p * (n - 2)**2/(n*(n - 1)) * x**2
             else:
                 k = (n - 1) // 2
                 R = C.RisingFactorial(S.Half, k)
@@ -723,10 +788,17 @@ class acosh(Function):
         else:
             return self.func(arg)
 
+    def inverse(self, argindex=1):
+        """
+        Returns the inverse of this function.
+        """
+        return cosh
+
     def _sage_(self):
         import sage.all as sage
         return sage.acosh(self.args[0]._sage_())
 
+
 class atanh(Function):
     """
     The inverse hyperbolic tangent function.
@@ -738,11 +810,10 @@ class atanh(Function):
 
     asinh, acosh, tanh
     """
-    nargs = 1
 
     def fdiff(self, argindex=1):
         if argindex == 1:
-            return 1/(1-self.args[0]**2)
+            return 1/(1 - self.args[0]**2)
         else:
             raise ArgumentIndexError(self, argindex)
 
@@ -794,21 +865,27 @@ class atanh(Function):
         else:
             return self.func(arg)
 
+    def inverse(self, argindex=1):
+        """
+        Returns the inverse of this function.
+        """
+        return tanh
+
     def _sage_(self):
         import sage.all as sage
         return sage.atanh(self.args[0]._sage_())
 
+
 class acoth(Function):
     """
     The inverse hyperbolic cotangent function.
 
     * acoth(x) -> Returns the inverse hyperbolic cotangent of x
     """
-    nargs = 1
 
     def fdiff(self, argindex=1):
         if argindex == 1:
-            return 1/(1-self.args[0]**2)
+            return 1/(1 - self.args[0]**2)
         else:
             raise ArgumentIndexError(self, argindex)
 
@@ -862,6 +939,12 @@ class acoth(Function):
         else:
             return self.func(arg)
 
+    def inverse(self, argindex=1):
+        """
+        Returns the inverse of this function.
+        """
+        return coth
+
     def _sage_(self):
         import sage.all as sage
         return sage.acoth(self.args[0]._sage_())
