@@ -1,13 +1,15 @@
+from __future__ import print_function, division
+
 __all__ = ['KanesMethod']
 
 from sympy import Symbol, zeros, Matrix, diff, solve_linear_system_LU, eye
+from sympy.core.compatibility import reduce
 from sympy.utilities import default_sort_key
-from sympy.physics.mechanics.essential import ReferenceFrame, dynamicsymbols
+from sympy.physics.vector import ReferenceFrame, dynamicsymbols, \
+     Point, partial_velocity
 from sympy.physics.mechanics.particle import Particle
-from sympy.physics.mechanics.point import Point
 from sympy.physics.mechanics.rigidbody import RigidBody
-from sympy.physics.mechanics.functions import (inertia_of_point_mass,
-                                               partial_velocity)
+from sympy.physics.mechanics.functions import inertia_of_point_mass
 
 class KanesMethod(object):
     """Kane's method object.
@@ -88,10 +90,11 @@ class KanesMethod(object):
         >>> forcing = KM.forcing
         >>> rhs = MM.inv() * forcing
         >>> rhs
-        [(-c*u(t) - k*q(t))/m]
+        Matrix([[(-c*u(t) - k*q(t))/m]])
         >>> KM.linearize()[0]
-        [ 0,  1]
-        [-k, -c]
+        Matrix([
+        [ 0,  1],
+        [-k, -c]])
 
     Please look at the documentation pages for more information on how to
     perform linearization and how to deal with dependent coordinates & speeds,
@@ -100,6 +103,9 @@ class KanesMethod(object):
     """
 
     simp = True
+    ___KDEqError = AttributeError('Create an instance of KanesMethod with' +
+                                  'kinematic differential equations to use' +
+                                  'this method.')
 
     def __init__(self, frame, q_ind, u_ind, kd_eqs=None, q_dependent=[],
             configuration_constraints=[], u_dependent=[],
@@ -127,12 +133,13 @@ class KanesMethod(object):
         self._udot = None
         self._uaux = None
 
-        # Differential Equations Matrices
+        # Differential Equations Matrices and Map
         self._k_d = None
         self._f_d = None
         self._k_kqdot = None
         self._k_ku = None
         self._f_k = None
+        self._qdot_u_map = None
 
         # Constraint Matrices
         self._f_h = Matrix([])
@@ -177,8 +184,8 @@ class KanesMethod(object):
 
         r1, c1 = A.shape
         r2, c2 = B.shape
-        temp1 = Matrix(r1, c1, lambda i, j: Symbol('x' + str(j + r1 * i)))
-        temp2 = Matrix(r2, c2, lambda i, j: Symbol('y' + str(j + r2 * i)))
+        temp1 = Matrix(r1, c1, lambda i, j: Symbol('x' + str(j) + str(r1 * i)))
+        temp2 = Matrix(r2, c2, lambda i, j: Symbol('y' + str(j) + str(r2 * i)))
         for i in range(len(temp1)):
             if A[i] == 0:
                 temp1[i] = 0
@@ -189,10 +196,10 @@ class KanesMethod(object):
         for i in range(c2):
             temp3.append(temp1.LDLsolve(temp2[:, i]))
         temp3 = Matrix([i.T for i in temp3]).T
-        return temp3.subs(dict(zip(temp1, A))).subs(dict(zip(temp2, B)))
+        return temp3.subs(dict(list(zip(temp1, A)))).subs(dict(list(zip(temp2, B))))
 
     def _coords(self, qind, qdep=[], coneqs=[]):
-        """Supply all the generalized coordiantes in a list.
+        """Supply all the generalized coordinates in a list.
 
         If some coordinates are dependent, supply them as part of qdep. Their
         dependent nature will only show up in the linearization process though.
@@ -261,22 +268,22 @@ class KanesMethod(object):
         if len(udep) != len(coneqs):
             raise ValueError('There must be an equal number of dependent '
                              'speeds and constraints.')
-        if diffconeqs != None:
+        if diffconeqs is not None:
             if len(udep) != len(diffconeqs):
                 raise ValueError('There must be an equal number of dependent '
                                  'speeds and constraints.')
         if len(udep) != 0:
             u = self._u
-            uzero = dict(zip(u, [0] * len(u)))
+            uzero = dict(list(zip(u, [0] * len(u))))
             coneqs = Matrix(coneqs)
             udot = self._udot
-            udotzero = dict(zip(udot, [0] * len(udot)))
+            udotzero = dict(list(zip(udot, [0] * len(udot))))
 
             self._udep = udep
             self._f_nh = coneqs.subs(uzero)
             self._k_nh = (coneqs - self._f_nh).jacobian(u)
             # if no differentiated non holonomic constraints were given, calculate
-            if diffconeqs == None:
+            if diffconeqs is None:
                 self._k_dnh = self._k_nh
                 self._f_dnh = (self._k_nh.diff(dynamicsymbols._t) * Matrix(u) +
                                self._f_nh.diff(dynamicsymbols._t))
@@ -284,30 +291,37 @@ class KanesMethod(object):
                 self._f_dnh = diffconeqs.subs(udotzero)
                 self._k_dnh = (diffconeqs - self._f_dnh).jacobian(udot)
 
-            o = len(u) # number of generalized speeds
-            m = len(udep) # number of motion constraints
-            p = o - m # number of independent speeds
+            o = len(u)  # number of generalized speeds
+            m = len(udep)  # number of motion constraints
+            p = o - m  # number of independent speeds
             # For a reminder, form of non-holonomic constraints is:
             # B u + C = 0
-            B = self._k_nh[:m, :]
-            C = self._f_nh[:m, 0]
+            B = self._k_nh[:, :]
+            C = self._f_nh[:, 0]
 
             # We partition B into indenpendent and dependent columns
             # Ars is then -Bdep.inv() * Bind, and it relates depedent speeds to
             # independent speeds as: udep = Ars uind, neglecting the C term here.
             self._depB = B
             self._depC = C
-            mr1 = B[:m, :p]
-            ml1 = B[:m, p:o]
+            mr1 = B[:, :p]
+            ml1 = B[:, p:o]
             self._Ars = - self._mat_inv_mul(ml1, mr1)
 
+    def _partial_velocity(self, vlist, ulist, frame):
+        """Returns the list of partial velocities, replacing qdot's in the
+        velocity list if necessary.
+        """
+        if self._qdot_u_map is None:
+            raise ___KDEqError
+        v = [vel.subs(self._qdot_u_map) for vel in vlist]
+        return partial_velocity(v, ulist, frame)
+
     def kindiffdict(self):
         """Returns the qdot's in a dictionary. """
-        if self._k_kqdot == None:
-            raise ValueError('Kin. diff. eqs need to be supplied first.')
-        sub_dict = solve_linear_system_LU(Matrix([self._k_kqdot.T,
-            -(self._k_ku * Matrix(self._u) + self._f_k).T]).T, self._qdot)
-        return sub_dict
+        if self._qdot_u_map is None:
+            raise ___KDEqError
+        return self._qdot_u_map
 
     def _kindiffeq(self, kdeqs):
         """Supply all the kinematic differential equations in a list.
@@ -328,14 +342,15 @@ class KanesMethod(object):
 
         uaux = self._uaux
         # dictionary of auxiliary speeds which are equal to zero
-        uaz = dict(zip(uaux, [0] * len(uaux)))
+        uaz = dict(list(zip(uaux, [0] * len(uaux))))
 
-        kdeqs = Matrix(kdeqs).subs(uaz)
+        #kdeqs = Matrix(kdeqs).subs(uaz)
+        kdeqs = Matrix(kdeqs)
 
         qdot = self._qdot
-        qdotzero = dict(zip(qdot, [0] * len(qdot)))
+        qdotzero = dict(list(zip(qdot, [0] * len(qdot))))
         u = self._u
-        uzero = dict(zip(u, [0] * len(u)))
+        uzero = dict(list(zip(u, [0] * len(u))))
 
         f_k = kdeqs.subs(uzero).subs(qdotzero)
         k_kqdot = (kdeqs.subs(uzero) - f_k).jacobian(Matrix(qdot))
@@ -344,6 +359,11 @@ class KanesMethod(object):
         self._k_ku = self._mat_inv_mul(k_kqdot, k_ku)
         self._f_k = self._mat_inv_mul(k_kqdot, f_k)
         self._k_kqdot = eye(len(qdot))
+        self._qdot_u_map = solve_linear_system_LU(Matrix([self._k_kqdot.T,
+            -(self._k_ku * Matrix(self._u) + self._f_k).T]).T, self._qdot)
+
+        self._k_ku = self._mat_inv_mul(k_kqdot, k_ku).subs(uaz)
+        self._f_k = self._mat_inv_mul(k_kqdot, f_k).subs(uaz)
 
     def _form_fr(self, fl):
         """Form the generalized active force.
@@ -366,8 +386,8 @@ class KanesMethod(object):
         N = self._inertial
         self._forcelist = fl[:]
         u = self._u
-        o = len(u) # number of gen. speeds
-        b = len(fl) # number of forces
+        o = len(u)  # number of gen. speeds
+        b = len(fl)  # number of forces
 
         FR = zeros(o, 1)
 
@@ -382,15 +402,15 @@ class KanesMethod(object):
             else:
                 raise TypeError('First entry in pair must be point or frame.')
             f_list += [i[1]]
-        partials = partial_velocity(vel_list, u, N)
+        partials = self._partial_velocity(vel_list, u, N)
 
         # Fill Fr with dot product of partial velocities and forces
         for i in range(o):
             for j in range(b):
-                FR[i] -= partials[j][i] & f_list[j]
+                FR[i] += partials[j][i] & f_list[j]
 
         # In case there are dependent speeds
-        m = len(self._udep) # number of dependent speeds
+        m = len(self._udep)  # number of dependent speeds
         if m != 0:
             p = o - m
             FRtilde = FR[:p, 0]
@@ -420,19 +440,24 @@ class KanesMethod(object):
         t = dynamicsymbols._t
         N = self._inertial
         self._bodylist = bl
-        u = self._u # all speeds
-        udep = self._udep # dependent speeds
+        u = self._u  # all speeds
+        udep = self._udep  # dependent speeds
         o = len(u)
         m = len(udep)
         p = o - m
         udot = self._udot
-        udotzero = dict(zip(udot, [0] * o))
+        udotzero = dict(list(zip(udot, [0] * o)))
         # auxiliary speeds
         uaux = self._uaux
         uauxdot = [diff(i, t) for i in uaux]
         # dictionary of auxiliary speeds which are equal to zero
-        uaz = dict(zip(uaux, [0] * len(uaux)))
-        uadz = dict(zip(uauxdot, [0] * len(uauxdot)))
+        uaz = dict(list(zip(uaux, [0] * len(uaux))))
+        uadz = dict(list(zip(uauxdot, [0] * len(uauxdot))))
+        # dictionary of qdot's to u's
+        qdots = dict(list(zip(list(self._qdot_u_map.keys()),
+                         list(self._qdot_u_map.values()))))
+        for k, v in list(qdots.items()):
+            qdots[k.diff(t)] = v.diff(t)
 
         MM = zeros(o, o)
         nonMM = zeros(o, 1)
@@ -445,10 +470,11 @@ class KanesMethod(object):
         # partial velocities.
         for v in bl:
             if isinstance(v, RigidBody):
-                partials += [partial_velocity([v.masscenter.vel(N),
-                                               v.frame.ang_vel_in(N)], u, N)]
+                partials += [self._partial_velocity([v.masscenter.vel(N),
+                                                     v.frame.ang_vel_in(N)],
+                                                    u, N)]
             elif isinstance(v, Particle):
-                partials += [partial_velocity([v.point.vel(N)], u, N)]
+                partials += [self._partial_velocity([v.point.vel(N)], u, N)]
             else:
                 raise TypeError('The body list needs RigidBody or '
                                 'Particle as list elements.')
@@ -460,53 +486,40 @@ class KanesMethod(object):
         for i, v in enumerate(bl):
             if isinstance(v, RigidBody):
                 M = v.mass.subs(uaz).doit()
-                I, P = v.inertia
-                if P != v.masscenter:
-                    # redefine I about the center of mass
-                    # have I S/O, want I S/S*
-                    # I S/O = I S/S* + I S*/O; I S/S* = I S/O - I S*/O
-                    f = v.frame
-                    d = v.masscenter.pos_from(P)
-                    I -= inertia_of_point_mass(M, d, f)
-                I = I.subs(uaz).doit()
+                vel = v.masscenter.vel(N).subs(uaz).doit()
+                acc = v.masscenter.acc(N).subs(udotzero).subs(uaz).doit()
+                inertial_force = (M.diff(t) * vel + M * acc)
+                omega = v.frame.ang_vel_in(N).subs(uaz).doit()
+                I = v.central_inertia.subs(uaz).doit()
+                inertial_torque = ((I.dt(v.frame) & omega).subs(uaz).doit() +
+                    (I & v.frame.ang_acc_in(N)).subs(udotzero).subs(uaz).doit() +
+                    (omega ^ (I & omega)).subs(uaz).doit())
                 for j in range(o):
+                    tmp_vel = partials[i][0][j].subs(uaz).doit()
+                    tmp_ang = (I & partials[i][1][j].subs(uaz).doit())
                     for k in range(o):
                         # translational
-                        MM[j, k] += M * (partials[i][0][j].subs(uaz).doit() &
-                                         partials[i][0][k])
+                        MM[j, k] += M * (tmp_vel & partials[i][0][k])
                         # rotational
-                        temp = (I & partials[i][1][j].subs(uaz).doit())
-                        MM[j, k] += (temp &
-                                     partials[i][1][k])
-                    # translational components
-                    nonMM[j] += ( (M.diff(t) *
-                                   v.masscenter.vel(N)).subs(uaz).doit() &
-                                 partials[i][0][j])
-                    nonMM[j] += (M *
-                            v.masscenter.acc(N).subs(udotzero).subs(uaz).doit()
-                            & partials[i][0][j])
-                    # rotational components
-                    omega = v.frame.ang_vel_in(N).subs(uaz).doit()
-                    nonMM[j] += ((I.dt(v.frame) & omega).subs(uaz).doit() &
-                                 partials[i][1][j])
-                    nonMM[j] += ((I &
-                        v.frame.ang_acc_in(N)).subs(udotzero).subs(uaz).doit()
-                        & partials[i][1][j])
-                    nonMM[j] += ((omega ^ (I & omega)).subs(uaz).doit() &
-                                 partials[i][1][j])
+                        MM[j, k] += (tmp_ang & partials[i][1][k])
+                    nonMM[j] += inertial_force & partials[i][0][j]
+                    nonMM[j] += inertial_torque & partials[i][1][j]
 
             if isinstance(v, Particle):
                 M = v.mass.subs(uaz).doit()
+                vel = v.point.vel(N).subs(uaz).doit()
+                acc = v.point.acc(N).subs(udotzero).subs(uaz).doit()
+                inertial_force = (M.diff(t) * vel + M * acc)
                 for j in range(o):
+                    temp = partials[i][0][j].subs(uaz).doit()
                     for k in range(o):
-                        MM[j, k] += M * (partials[i][0][j].subs(uaz).doit() &
-                                         partials[i][0][k])
-                    nonMM[j] += M.diff(t) * (v.point.vel(N).subs(uaz).doit() &
-                                             partials[i][0][j])
-                    nonMM[j] += (M *
-                            v.point.acc(N).subs(udotzero).subs(uaz).doit() &
-                            partials[i][0][j])
-        FRSTAR = MM * Matrix(udot).subs(uadz) + nonMM
+                        MM[j, k] += M * (temp & partials[i][0][k])
+                    nonMM[j] += inertial_force & partials[i][0][j]
+        # Negate FRSTAR since Kane defines the inertia forces/torques
+        # to be negative and we didn't do so above.
+        MM = MM.subs(qdots).subs(uaz).doit()
+        nonMM = nonMM.subs(qdots).subs(udotzero).subs(uadz).subs(uaz).doit()
+        FRSTAR = -(MM * Matrix(udot).subs(uadz) + nonMM)
 
         # For motion constraints, m is the number of constraints
         # Really, one should just look at Kane's book for descriptions of this
@@ -522,7 +535,7 @@ class KanesMethod(object):
             MM = MMi + self._Ars.T * MMd
         self._frstar = FRSTAR
 
-        zeroeq = self._fr + self._frstar
+        zeroeq = -(self._fr + self._frstar)
         zeroeq = zeroeq.subs(udotzero)
 
         self._k_d = MM
@@ -550,17 +563,24 @@ class KanesMethod(object):
 
         """
 
-        if (self._q == None) or (self._u == None):
+        if (self._q is None) or (self._u is None):
             raise ValueError('Speeds and coordinates must be supplied first.')
-        if (self._k_kqdot == None):
-            raise ValueError('Supply kinematic differential equations, please.')
+        if (self._k_kqdot is None):
+            raise __KDEqError
 
 
         fr = self._form_fr(FL)
         frstar = self._form_frstar(BL)
         if self._uaux != []:
-            km = KanesMethod(self._inertial, self._q, self._uaux,
+            if self._udep == []:
+                km = KanesMethod(self._inertial, self._q, self._uaux,
                              u_auxiliary=self._uaux)
+            else:
+                km = KanesMethod(self._inertial, self._q, self._uaux,
+                u_auxiliary=self._uaux, u_dependent=self._udep,
+                velocity_constraints=(self._k_nh * Matrix(self._u) + self._f_nh))
+            km._qdot_u_map = self._qdot_u_map
+            self._km = km
             fraux = km._form_fr(FL)
             frstaraux = km._form_frstar(BL)
             self._aux_eq = fraux + frstaraux
@@ -603,7 +623,7 @@ class KanesMethod(object):
 
         """
 
-        if (self._fr == None) or (self._frstar == None):
+        if (self._fr is None) or (self._frstar is None):
             raise ValueError('Need to compute Fr, Fr* first.')
 
         # Note that this is now unneccessary, and it should never be
@@ -617,11 +637,12 @@ class KanesMethod(object):
         uaux = self._uaux
         uauxdot = [diff(i, t) for i in uaux]
         # dictionary of auxiliary speeds & derivatives which are equal to zero
-        subdict = dict(zip(uaux + uauxdot, [0] * (len(uaux) + len(uauxdot))))
+        subdict = dict(list(zip(uaux + uauxdot, [0] * (len(uaux) + len(uauxdot)))))
 
         # Checking for dynamic symbols outside the dynamic differential
         # equations; throws error if there is.
-        insyms = set(self._q + self._qdot + self._u + self._udot + uaux + uauxdot)
+        insyms = set(
+            self._q + self._qdot + self._u + self._udot + uaux + uauxdot)
         if any(self._find_dynamicsymbols(i, insyms) for i in [self._k_kqdot,
                                                               self._k_ku,
                                                               self._f_k,
@@ -641,15 +662,15 @@ class KanesMethod(object):
                 raise ValueError('Cannot have derivatives of specified '
                                  'quantities when linearizing forcing terms.')
 
-        o = len(self._u) # number of speeds
-        n = len(self._q) # number of coordinates
-        l = len(self._qdep) # number of configuration constraints
-        m = len(self._udep) # number of motion constraints
-        qi = Matrix(self._q[: n - l]) # independent coords
-        qd = Matrix(self._q[n - l : n]) # dependent coords; could be empty
-        ui = Matrix(self._u[: o - m]) # independent speeds
-        ud = Matrix(self._u[o - m : o]) # dependent speeds; could be empty
-        qdot = Matrix(self._qdot) # time derivatives of coordinates
+        o = len(self._u)  # number of speeds
+        n = len(self._q)  # number of coordinates
+        l = len(self._qdep)  # number of configuration constraints
+        m = len(self._udep)  # number of motion constraints
+        qi = Matrix(self._q[: n - l])  # independent coords
+        qd = Matrix(self._q[n - l: n])  # dependent coords; could be empty
+        ui = Matrix(self._u[: o - m])  # independent speeds
+        ud = Matrix(self._u[o - m: o])  # dependent speeds; could be empty
+        qdot = Matrix(self._qdot)  # time derivatives of coordinates
 
         # with equations in the form MM udot = forcing, expand that to:
         # MM_full [q,u].T = forcing_full. This combines coordinates and
@@ -731,7 +752,7 @@ class KanesMethod(object):
             dqdot_dui = - self._k_kqdot.inv() * fku.jacobian(ui)
             dqdot_dqi = - self._k_kqdot.inv() * (fku.jacobian(qi) +
                 fkf.jacobian(qi) + (fku.jacobian(qd) + fkf.jacobian(qd)) *
-                    dqd_dqi)
+                dqd_dqi)
             f1_q = (f1.jacobian(qi) + f1.jacobian(qd) * dqd_dqi)
             f1_u = f1.jacobian(ui)
             f2_jac_qdot = f2.jacobian(qdot)
@@ -791,8 +812,7 @@ class KanesMethod(object):
             The specific sympy inverse matrix calculation method to use.
 
         """
-
-        if inv_method == None:
+        if inv_method is None:
             self._rhs = self._mat_inv_mul(self.mass_matrix_full,
                                           self.forcing_full)
         else:
@@ -802,7 +822,7 @@ class KanesMethod(object):
 
     @property
     def auxiliary_eqs(self):
-        if (self._fr == None) or (self._frstar == None):
+        if (self._fr is None) or (self._frstar is None):
             raise ValueError('Need to compute Fr, Fr* first.')
         if self._uaux == []:
             raise ValueError('No auxiliary speeds have been declared.')
@@ -812,14 +832,14 @@ class KanesMethod(object):
     def mass_matrix(self):
         # Returns the mass matrix, which is augmented by the differentiated non
         # holonomic equations if necessary
-        if (self._frstar == None) & (self._fr == None):
+        if (self._frstar is None) & (self._fr is None):
             raise ValueError('Need to compute Fr, Fr* first.')
         return Matrix([self._k_d, self._k_dnh])
 
     @property
     def mass_matrix_full(self):
         # Returns the mass matrix from above, augmented by kin diff's k_kqdot
-        if (self._frstar == None) & (self._fr == None):
+        if (self._frstar is None) & (self._fr is None):
             raise ValueError('Need to compute Fr, Fr* first.')
         o = len(self._u)
         n = len(self._q)
@@ -830,7 +850,7 @@ class KanesMethod(object):
     def forcing(self):
         # Returns the forcing vector, which is augmented by the differentiated
         # non holonomic equations if necessary
-        if (self._frstar == None) & (self._fr == None):
+        if (self._frstar is None) & (self._fr is None):
             raise ValueError('Need to compute Fr, Fr* first.')
         return -Matrix([self._f_d, self._f_dnh])
 
@@ -838,7 +858,7 @@ class KanesMethod(object):
     def forcing_full(self):
         # Returns the forcing vector, which is augmented by the differentiated
         # non holonomic equations if necessary
-        if (self._frstar == None) & (self._fr == None):
+        if (self._frstar is None) & (self._fr is None):
             raise ValueError('Need to compute Fr, Fr* first.')
         f1 = self._k_ku * Matrix(self._u) + self._f_k
         return -Matrix([f1, self._f_d, self._f_dnh])
