@@ -12,8 +12,14 @@ sympy.stats.frv
 sympy.stats.rv_interface
 """
 
-from sympy import Basic, S, Expr, Symbol, Tuple, And, Add, Eq, lambdify
+from __future__ import print_function, division
+
+from sympy import (Basic, S, Expr, Symbol, Tuple, And, Add, Eq, lambdify,
+        sympify, Equality, solve, Lambda, DiracDelta)
+from sympy.core.compatibility import reduce
 from sympy.core.sets import FiniteSet, ProductSet
+from sympy.abc import x
+
 
 class RandomDomain(Basic):
     """
@@ -47,6 +53,7 @@ class RandomDomain(Basic):
     def integrate(self, expr):
         raise NotImplementedError()
 
+
 class SingleDomain(RandomDomain):
     """
     A single variable and its domain
@@ -58,19 +65,23 @@ class SingleDomain(RandomDomain):
     """
     def __new__(cls, symbol, set):
         assert symbol.is_Symbol
-        symbols = FiniteSet(symbol)
-        return RandomDomain.__new__(cls, symbols, set)
+        return Basic.__new__(cls, symbol, set)
 
     @property
     def symbol(self):
-        return tuple(self.symbols)[0]
+        return self.args[0]
+
+    @property
+    def symbols(self):
+        return FiniteSet(self.symbol)
 
     def __contains__(self, other):
-        if len(other)!=1:
+        if len(other) != 1:
             return False
         sym, val = tuple(other)[0]
         return self.symbol == sym and val in self.set
 
+
 class ConditionalDomain(RandomDomain):
     """
     A RandomDomain with an attached condition
@@ -81,18 +92,21 @@ class ConditionalDomain(RandomDomain):
     sympy.stats.frv.ConditionalFiniteDomain
     """
     def __new__(cls, fulldomain, condition):
-        condition = condition.subs(dict((rs,rs.symbol)
+        condition = condition.xreplace(dict((rs, rs.symbol)
             for rs in random_symbols(condition)))
-        return RandomDomain.__new__(
-                cls, fulldomain.symbols, fulldomain, condition)
+        return Basic.__new__(cls, fulldomain, condition)
+
+    @property
+    def symbols(self):
+        return self.fulldomain.symbols
 
     @property
     def fulldomain(self):
-        return self.args[1]
+        return self.args[0]
 
     @property
     def condition(self):
-        return self.args[2]
+        return self.args[1]
 
     @property
     def set(self):
@@ -101,6 +115,7 @@ class ConditionalDomain(RandomDomain):
     def as_boolean(self):
         return And(self.fulldomain.as_boolean(), self.condition)
 
+
 class PSpace(Basic):
     """
     A Probability Space
@@ -117,6 +132,7 @@ class PSpace(Basic):
 
     is_Finite = None
     is_Continuous = None
+    is_real = None
 
     @property
     def domain(self):
@@ -149,17 +165,37 @@ class PSpace(Basic):
     def integrate(self, expr):
         raise NotImplementedError()
 
+
 class SinglePSpace(PSpace):
     """
     Represents the probabilities of a set of random events that can be
     attributed to a single variable/symbol.
     """
+    def __new__(cls, s, distribution):
+        if isinstance(s, str):
+            s = Symbol(s)
+        if not isinstance(s, Symbol):
+            raise TypeError("s should have been string or Symbol")
+        return Basic.__new__(cls, s, distribution)
 
     @property
     def value(self):
-        return tuple(self.values)[0]
+        return RandomSymbol(self, self.symbol)
 
-class RandomSymbol(Symbol):
+    @property
+    def symbol(self):
+        return self.args[0]
+
+    @property
+    def distribution(self):
+        return self.args[1]
+
+    @property
+    def pdf(self):
+        return self.distribution.pdf(self.symbol)
+
+
+class RandomSymbol(Expr):
     """
     Random Symbols represent ProbabilitySpaces in SymPy Expressions
     In principle they can take on any value that their symbol can take on
@@ -185,18 +221,27 @@ class RandomSymbol(Symbol):
     convenience functions Normal, Exponential, Coin, Die, FiniteRV, etc....
     """
 
-    is_bounded=True
-    is_finite=True
+    def __new__(cls, pspace, symbol):
+        if not isinstance(symbol, Symbol):
+            raise TypeError("symbol should be of type Symbol")
+        if not isinstance(pspace, PSpace):
+            raise TypeError("pspace variable should be of type PSpace")
+        return Basic.__new__(cls, pspace, symbol)
 
-    def __new__(cls, *args):
-        obj = Basic.__new__(cls)
-        obj.pspace = args[0]
-        obj.symbol = args[1]
-        return obj
+    is_bounded = True
+    is_finite = True
+    is_Symbol = True
+    is_Atom = True
 
-    @property
-    def name(self):
-        return self.symbol.name
+    _diff_wrt = True
+
+    pspace = property(lambda self: self.args[0])
+    symbol = property(lambda self: self.args[1])
+    name   = property(lambda self: self.symbol.name)
+
+    is_positive = property(lambda self: self.symbol.is_positive)
+    is_integer = property(lambda self: self.symbol.is_integer)
+    is_real = property(lambda self: self.symbol.is_real or self.pspace.is_real)
 
     @property
     def is_commutative(self):
@@ -205,6 +250,11 @@ class RandomSymbol(Symbol):
     def _hashable_content(self):
         return self.pspace, self.symbol
 
+    @property
+    def free_symbols(self):
+        return set([self])
+
+
 class ProductPSpace(PSpace):
     """
     A probability space resulting from the merger of two independent probability
@@ -232,14 +282,25 @@ class ProductPSpace(PSpace):
             from sympy.stats.crv import ProductContinuousPSpace
             cls = ProductContinuousPSpace
 
-        obj = Basic.__new__(cls, symbols, FiniteSet(*spaces))
-        obj.rs_space_dict = rs_space_dict
+        obj = Basic.__new__(cls, *FiniteSet(*spaces))
 
         return obj
 
+    @property
+    def rs_space_dict(self):
+        d = {}
+        for space in self.spaces:
+            for value in space.values:
+                d[value] = space
+        return d
+
+    @property
+    def symbols(self):
+        return FiniteSet(val.symbol for val in self.rs_space_dict.keys())
+
     @property
     def spaces(self):
-        return self.args[1]
+        return FiniteSet(*self.args)
 
     @property
     def values(self):
@@ -261,8 +322,9 @@ class ProductPSpace(PSpace):
         raise NotImplementedError("Density not available for ProductSpaces")
 
     def sample(self):
-        return dict([(k,v) for space in self.spaces
-            for k,v in space.sample().items()])
+        return dict([(k, v) for space in self.spaces
+            for k, v in space.sample().items()])
+
 
 class ProductDomain(RandomDomain):
     """
@@ -287,11 +349,6 @@ class ProductDomain(RandomDomain):
                 domains2.extend(domain.domains)
         domains2 = FiniteSet(domains2)
 
-        sym_domain_dict = {}
-        for domain in domains2:
-            for symbol in domain.symbols:
-                sym_domain_dict[symbol] = domain
-
         if all(domain.is_Finite for domain in domains2):
             from sympy.stats.frv import ProductFiniteDomain
             cls = ProductFiniteDomain
@@ -299,13 +356,21 @@ class ProductDomain(RandomDomain):
             from sympy.stats.crv import ProductContinuousDomain
             cls = ProductContinuousDomain
 
-        obj = RandomDomain.__new__(cls, symbols, domains2)
-        obj.sym_domain_dict = sym_domain_dict
-        return obj
+        return Basic.__new__(cls, *domains2)
+
+    @property
+    def sym_domain_dict(self):
+        return dict((symbol, domain) for domain in self.domains
+                                     for symbol in domain.symbols)
+
+    @property
+    def symbols(self):
+        return FiniteSet(sym for domain in self.domains
+                             for sym    in domain.symbols)
 
     @property
     def domains(self):
-        return self.args[1]
+        return self.args
 
     @property
     def set(self):
@@ -326,6 +391,7 @@ class ProductDomain(RandomDomain):
     def as_boolean(self):
         return And(*[domain.as_boolean() for domain in self.domains])
 
+
 def random_symbols(expr):
     """
     Returns all RandomSymbols within a SymPy Expression.
@@ -335,6 +401,7 @@ def random_symbols(expr):
     except AttributeError:
         return []
 
+
 def pspace(expr):
     """
     Returns the underlying Probability Space of a random expression.
@@ -360,32 +427,35 @@ def pspace(expr):
     # Otherwise make a product space
     return ProductPSpace(*[rv.pspace for rv in rvs])
 
+
 def sumsets(sets):
     """
     Union of sets
     """
     return reduce(frozenset.union, sets, frozenset())
 
-def rs_swap(a,b):
+
+def rs_swap(a, b):
     """
     Build a dictionary to swap RandomSymbols based on their underlying symbol.
 
     i.e.
-    if    X = ('x', pspace1)
-    and   Y = ('x', pspace2)
-    then X and Y match and the key, value pair
-    {X:Y} will appear in the result
+    if    ``X = ('x', pspace1)``
+    and   ``Y = ('x', pspace2)``
+    then ``X`` and ``Y`` match and the key, value pair
+    ``{X:Y}`` will appear in the result
 
     Inputs: collections a and b of random variables which share common symbols
     Output: dict mapping RVs in a to RVs in b
     """
     d = {}
     for rsa in a:
-        d[rsa] = [rsb for rsb in b if rsa.symbol==rsb.symbol][0]
+        d[rsa] = [rsb for rsb in b if rsa.symbol == rsb.symbol][0]
     return d
 
+
 def given(expr, condition=None, **kwargs):
-    """
+    """ Conditional Random Expression
     From a random expression and a condition on that expression creates a new
     probability space from the condition and returns the same expression on that
     conditional probability space.
@@ -396,13 +466,42 @@ def given(expr, condition=None, **kwargs):
     >>> from sympy.stats import given, density, Die
     >>> X = Die('X', 6)
     >>> Y = given(X, X>3)
-    >>> density(Y)
+    >>> density(Y).dict
     {4: 1/3, 5: 1/3, 6: 1/3}
+
+    Following convention, if the condition is a random symbol then that symbol
+    is considered fixed.
+
+    >>> from sympy.stats import Normal
+    >>> from sympy import pprint
+    >>> from sympy.abc import z
+
+    >>> X = Normal('X', 0, 1)
+    >>> Y = Normal('Y', 0, 1)
+    >>> pprint(density(X + Y, Y)(z), use_unicode=False)
+                    2
+           -(-Y + z)
+           -----------
+      ___       2
+    \/ 2 *e
+    ------------------
+             ____
+         2*\/ pi
     """
 
     if not random_symbols(condition) or pspace_independent(expr, condition):
         return expr
 
+    if isinstance(condition, RandomSymbol):
+        condition = Eq(condition, condition.symbol)
+
+    condsymbols = random_symbols(condition)
+    if (isinstance(condition, Equality) and len(condsymbols) == 1 and
+        not isinstance(pspace(expr).domain, ConditionalDomain)):
+        rv = tuple(condsymbols)[0]
+        results = solve(condition, rv)
+        return sum(expr.subs(rv, res) for res in results)
+
     # Get full probability space of both the expression and the condition
     fullspace = pspace(Tuple(expr, condition))
     # Build new space given the condition
@@ -411,10 +510,11 @@ def given(expr, condition=None, **kwargs):
     # That point to the new conditional space
     swapdict = rs_swap(fullspace.values, space.values)
     # Swap random variables in the expression
-    expr = expr.subs(swapdict)
+    expr = expr.xreplace(swapdict)
     return expr
 
-def expectation(expr, condition=None, numsamples=None, **kwargs):
+
+def expectation(expr, condition=None, numsamples=None, evaluate=True, **kwargs):
     """
     Returns the expected value of a random expression
 
@@ -445,24 +545,31 @@ def expectation(expr, condition=None, numsamples=None, **kwargs):
     5
     """
 
-    if not random_symbols(expr): # expr isn't random?
+    if not random_symbols(expr):  # expr isn't random?
         return expr
-    if numsamples: # Computing by monte carlo sampling?
-        return sampling_E(expr, condition, numsamples=numsamples, **kwargs)
+    if numsamples:  # Computing by monte carlo sampling?
+        return sampling_E(expr, condition, numsamples=numsamples)
 
     # Create new expr and recompute E
-    if condition is not None: # If there is a condition
-        return expectation(given(expr, condition, **kwargs), **kwargs)
+    if condition is not None:  # If there is a condition
+        return expectation(given(expr, condition), evaluate=evaluate)
 
     # A few known statements for efficiency
 
-    if expr.is_Add: # We know that E is Linear
-        return Add(*[expectation(arg, **kwargs) for arg in expr.args])
+    if expr.is_Add:  # We know that E is Linear
+        return Add(*[expectation(arg, evaluate=evaluate)
+                     for arg in expr.args])
 
     # Otherwise case is simple, pass work off to the ProbabilitySpace
-    return pspace(expr).integrate(expr, **kwargs)
+    result = pspace(expr).integrate(expr)
+    if evaluate and hasattr(result, 'doit'):
+        return result.doit(**kwargs)
+    else:
+        return result
 
-def probability(condition, given_condition=None, numsamples=None,  **kwargs):
+
+def probability(condition, given_condition=None, numsamples=None,
+                evaluate=True, **kwargs):
     """
     Probability that a condition is true, optionally given a second condition
 
@@ -496,14 +603,48 @@ def probability(condition, given_condition=None, numsamples=None,  **kwargs):
     if numsamples:
         return sampling_P(condition, given_condition, numsamples=numsamples,
                 **kwargs)
-    if given_condition is not None: # If there is a condition
+    if given_condition is not None:  # If there is a condition
         # Recompute on new conditional expr
-        return probability(given(condition, given_condition, **kwargs),**kwargs)
+        return probability(given(condition, given_condition, **kwargs), **kwargs)
 
     # Otherwise pass work off to the ProbabilitySpace
-    return pspace(condition).probability(condition, **kwargs)
+    result = pspace(condition).probability(condition, **kwargs)
+    if evaluate and hasattr(result, 'doit'):
+        return result.doit()
+    else:
+        return result
 
-def density(expr, condition=None, **kwargs):
+
+class Density(Basic):
+    expr = property(lambda self: self.args[0])
+
+    @property
+    def condition(self):
+        if len(self.args) > 1:
+            return self.args[1]
+        else:
+            return None
+
+    def doit(self, evaluate=True, **kwargs):
+        expr, condition = self.expr, self.condition
+        if condition is not None:
+            # Recompute on new conditional expr
+            expr = given(expr, condition, **kwargs)
+        if not random_symbols(expr):
+            return Lambda(x, DiracDelta(x-expr))
+        if (isinstance(expr, RandomSymbol) and
+            hasattr(expr.pspace, 'distribution') and
+            isinstance(pspace(expr), SinglePSpace)):
+            return expr.pspace.distribution
+        result = pspace(expr).compute_density(expr, **kwargs)
+
+        if evaluate and hasattr(result, 'doit'):
+            return result.doit()
+        else:
+            return result
+
+
+def density(expr, condition=None, evaluate=True, **kwargs):
     """
     Probability density of a random expression
 
@@ -520,24 +661,21 @@ def density(expr, condition=None, **kwargs):
     >>> from sympy.stats import density, Die, Normal
     >>> from sympy import Symbol
 
+    >>> x = Symbol('x')
     >>> D = Die('D', 6)
-    >>> X = Normal('x', 0, 1)
+    >>> X = Normal(x, 0, 1)
 
-    >>> density(D)
+    >>> density(D).dict
     {1: 1/6, 2: 1/6, 3: 1/6, 4: 1/6, 5: 1/6, 6: 1/6}
-    >>> density(2*D)
+    >>> density(2*D).dict
     {2: 1/6, 4: 1/6, 6: 1/6, 8: 1/6, 10: 1/6, 12: 1/6}
-    >>> density(X)
-    Lambda(_x, sqrt(2)*exp(-_x**2/2)/(2*sqrt(pi)))
+    >>> density(X)(x)
+    sqrt(2)*exp(-x**2/2)/(2*sqrt(pi))
     """
-    if condition is not None: # If there is a condition
-        # Recompute on new conditional expr
-        return density(given(expr, condition, **kwargs), **kwargs)
+    return Density(expr, condition).doit(evaluate=evaluate, **kwargs)
 
-    # Otherwise pass work off to the ProbabilitySpace
-    return pspace(expr).compute_density(expr, **kwargs)
 
-def cdf(expr, condition=None, **kwargs):
+def cdf(expr, condition=None, evaluate=True, **kwargs):
     """
     Cumulative Distribution Function of a random expression.
 
@@ -557,7 +695,7 @@ def cdf(expr, condition=None, **kwargs):
     >>> D = Die('D', 6)
     >>> X = Normal('X', 0, 1)
 
-    >>> density(D)
+    >>> density(D).dict
     {1: 1/6, 2: 1/6, 3: 1/6, 4: 1/6, 5: 1/6, 6: 1/6}
     >>> cdf(D)
     {1: 1/6, 2: 1/3, 3: 1/2, 4: 2/3, 5: 5/6, 6: 1}
@@ -567,12 +705,18 @@ def cdf(expr, condition=None, **kwargs):
     >>> cdf(X)
     Lambda(_z, erf(sqrt(2)*_z/2)/2 + 1/2)
     """
-    if condition is not None: # If there is a condition
+    if condition is not None:  # If there is a condition
         # Recompute on new conditional expr
         return cdf(given(expr, condition, **kwargs), **kwargs)
 
     # Otherwise pass work off to the ProbabilitySpace
-    return pspace(expr).compute_cdf(expr, **kwargs)
+    result = pspace(expr).compute_cdf(expr, **kwargs)
+
+    if evaluate and hasattr(result, 'doit'):
+        return result.doit()
+    else:
+        return result
+
 
 def where(condition, given_condition=None, **kwargs):
     """
@@ -597,13 +741,14 @@ def where(condition, given_condition=None, **kwargs):
     >>> where(And(D1<=D2 , D2<3))
     Domain: Or(And(a == 1, b == 1), And(a == 1, b == 2), And(a == 2, b == 2))
     """
-    if given_condition is not None: # If there is a condition
+    if given_condition is not None:  # If there is a condition
         # Recompute on new conditional expr
         return where(given(condition, given_condition, **kwargs), **kwargs)
 
     # Otherwise pass work off to the ProbabilitySpace
     return pspace(condition).where(condition, **kwargs)
 
+
 def sample(expr, condition=None, **kwargs):
     """
     A realization of the random expression
@@ -616,7 +761,8 @@ def sample(expr, condition=None, **kwargs):
 
     >>> die_roll = sample(X+Y+Z) # A random realization of three dice
     """
-    return sample_iter(expr, condition, numsamples=1).next()
+    return next(sample_iter(expr, condition, numsamples=1))
+
 
 def sample_iter(expr, condition=None, numsamples=S.Infinity, **kwargs):
     """
@@ -650,6 +796,7 @@ def sample_iter(expr, condition=None, numsamples=S.Infinity, **kwargs):
     except TypeError:
         return sample_iter_subs(expr, condition, numsamples, **kwargs)
 
+
 def sample_iter_lambdify(expr, condition=None, numsamples=S.Infinity, **kwargs):
     """
     See sample_iter
@@ -669,7 +816,7 @@ def sample_iter_lambdify(expr, condition=None, numsamples=S.Infinity, **kwargs):
     # Check that lambdify can handle the expression
     # Some operations like Sum can prove difficult
     try:
-        d = ps.sample() # a dictionary that maps RVs to values
+        d = ps.sample()  # a dictionary that maps RVs to values
         args = [d[rv] for rv in rvs]
         fn(*args)
         if condition:
@@ -680,27 +827,29 @@ def sample_iter_lambdify(expr, condition=None, numsamples=S.Infinity, **kwargs):
     def return_generator():
         count = 0
         while count < numsamples:
-            d = ps.sample() # a dictionary that maps RVs to values
+            d = ps.sample()  # a dictionary that maps RVs to values
             args = [d[rv] for rv in rvs]
 
-            if condition: # Check that these values satisfy the condition
+            if condition:  # Check that these values satisfy the condition
                 gd = given_fn(*args)
-                if not isinstance(gd, bool):
-                    raise ValueError("Conditions must not contain free symbols")
-                if gd == False: # If the values don't satisfy then try again
+                if gd != True and gd != False:
+                    raise ValueError(
+                        "Conditions must not contain free symbols")
+                if not gd:  # If the values don't satisfy then try again
                     continue
 
             yield fn(*args)
             count += 1
     return return_generator()
 
+
 def sample_iter_subs(expr, condition=None, numsamples=S.Infinity, **kwargs):
     """
     See sample_iter
 
     Uses subs for computation. This is slow but almost always works.
     """
-    if condition:
+    if condition is not None:
         ps = pspace(Tuple(expr, condition))
     else:
         ps = pspace(expr)
@@ -708,19 +857,20 @@ def sample_iter_subs(expr, condition=None, numsamples=S.Infinity, **kwargs):
     count = 0
 
     while count < numsamples:
-        d = ps.sample() # a dictionary that maps RVs to values
+        d = ps.sample()  # a dictionary that maps RVs to values
 
-
-        if condition: # Check that these values satisfy the condition
-            gd = condition.subs(d)
-            if not isinstance(gd, bool):
+        if condition is not None:  # Check that these values satisfy the condition
+            gd = condition.xreplace(d)
+            if gd != True and gd != False:
                 raise ValueError("Conditions must not contain free symbols")
-            if gd == False: # If the values don't satisfy then try again
+            if not gd:  # If the values don't satisfy then try again
                 continue
 
-        yield expr.subs(d)
+        yield expr.xreplace(d)
 
         count += 1
+
+
 def sampling_P(condition, given_condition=None, numsamples=1,
                evalf=True, **kwargs):
     """
@@ -739,10 +889,10 @@ def sampling_P(condition, given_condition=None, numsamples=1,
                           numsamples=numsamples, **kwargs)
 
     for x in samples:
-        if not isinstance(x, bool):
+        if x != True and x != False:
             raise ValueError("Conditions must not contain free symbols")
 
-        if x==True:
+        if x:
             count_true += 1
         else:
             count_false += 1
@@ -753,6 +903,7 @@ def sampling_P(condition, given_condition=None, numsamples=1,
     else:
         return result
 
+
 def sampling_E(condition, given_condition=None, numsamples=1,
                evalf=True, **kwargs):
     """
@@ -773,6 +924,7 @@ def sampling_E(condition, given_condition=None, numsamples=1,
     else:
         return result
 
+
 def dependent(a, b):
     """
     Dependence of two random expressions
@@ -799,7 +951,7 @@ def dependent(a, b):
     ========
     independent
     """
-    if pspace_independent(a,b):
+    if pspace_independent(a, b):
         return False
 
     z = Symbol('z', real=True)
@@ -808,6 +960,7 @@ def dependent(a, b):
     return (density(a, Eq(b, z)) != density(a) or
             density(b, Eq(a, z)) != density(b))
 
+
 def independent(a, b):
     """
     Independence of two random expressions
@@ -836,7 +989,8 @@ def independent(a, b):
     """
     return not dependent(a, b)
 
-def pspace_independent(a,b):
+
+def pspace_independent(a, b):
     """
     Tests for independence between a and b by checking if their PSpaces have
     overlapping symbols. This is a sufficient but not necessary condition for
@@ -851,6 +1005,7 @@ def pspace_independent(a,b):
         return True
     return None
 
+
 def rv_subs(expr, symbols=None):
     """
     Given a random expression replace all random variables with their symbols.
@@ -859,5 +1014,26 @@ def rv_subs(expr, symbols=None):
     """
     if symbols is None:
         symbols = random_symbols(expr)
+    if not symbols:
+        return expr
     swapdict = dict([(rv, rv.symbol) for rv in symbols])
-    return expr.subs(swapdict)
+    return expr.xreplace(swapdict)
+
+class NamedArgsMixin(object):
+    _argnames = ()
+
+    def __getattr__(self, attr):
+        try:
+            return self.args[self._argnames.index(attr)]
+        except ValueError:
+            raise AttributeError("'%s' object has not attribute '%s'" % (
+                type(self).__name__, attr))
+
+def _value_check(condition, message):
+    """
+    Check a condition on input value.
+
+    Raises ValueError with message if condition is not True
+    """
+    if condition != True:
+        raise ValueError(message)
